/**
 * Copyright (c) 2010-2012 EBM WebSourcing, 2012-2013 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see <http://www.gnu.org/licenses/>
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.bc.mail.service.provide;

import java.util.logging.Level;
import java.util.logging.Logger;

import javax.jbi.JBIException;
import javax.jbi.messaging.MessagingException;
import javax.mail.Session;
import javax.mail.internet.MimeMessage;

import org.ow2.petals.bc.mail.MailSessionManager;
import org.ow2.petals.bc.mail.MailUtil;
import org.ow2.petals.bc.mail.MimeMessageManager;
import org.ow2.petals.bc.mail.MissingElementException;
import org.ow2.petals.component.framework.api.configuration.ConfigurationExtensions;
import org.ow2.petals.component.framework.api.message.Exchange;
import org.w3c.dom.Node;

/**
 * @author Mathieu CARROLLE - EBM WebSourcing
 */
public class SuSendService extends AbstractSendService {

    private final ConfigurationExtensions extensions;

    public SuSendService(MailSessionManager mailSessionManager,
            MimeMessageManager mimeMessageManager, Logger logger, ConfigurationExtensions extensions) {
        super(mailSessionManager, mimeMessageManager, logger);
        this.extensions = extensions;
    }

    /**
     * Standard service : send the jbi message content as an email<br>
     * the email address and subject are defined in the service unit
     * 
     * @param exchange
     * @throws JBIException
     */
    public void sendMail(Exchange exchange) throws MessagingException, MissingElementException {
        try {
            this.logger.log(Level.INFO, "JBI Exchange received - send it as a mail");
            // Build a mail session descriptor from the given address (URI)
            ProvideDescriptor provideDescriptor;

            provideDescriptor = MailUtil.build(extensions);

            // Build the mail session
            final Session session = this.mailSessionManager
                    .createSessionPropertiesFromDescriptor(provideDescriptor);
            Node rootNode = exchange.getInMessageContentAsDocument(false).getFirstChild();

            // find the send/mail xml element
            if (rootNode == null || !rootNode.getLocalName().equalsIgnoreCase("mail")) {
                throw new MissingElementException("mail");
            }
            
            // Create a mail message with the incoming jbi message
            final MimeMessage mimeMessage = this.mimeMessageManager
                    .mapNormalizedMessageToMimeMessage(session, provideDescriptor, exchange,
                            rootNode);

            // Send this mail
            this.mailSessionManager.sendMail(mimeMessage, provideDescriptor, session);

            this.logger.log(Level.INFO, "email sent to " + provideDescriptor.getToAddress());
        } catch (javax.mail.MessagingException e) {
            throw new MessagingException(e);
        }
    }
}
